/* $Id: download.cc,v 1.8 2002/01/01 19:00:57 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2001 Richard Atterer
  | \/|  <richard@atterer.net>
   '` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

  Download data from URL, write to output function, report on progress

*/

#include <string.h>

#include <config.h>
#include <debug.hh>
#include <download.hh>
//#include <download.ih>
#include <string.hh>
//______________________________________________________________________

Download::Download(const string& uri, Output* o)
    : uriVal(uri), totalSize(0), output(o), request(0) {
  static const HTStreamClass downloadWriter = {
    "jigdoDownloadWriter", flush, free, abort, putChar, putString, write
  };
  vptr = &downloadWriter;
}

Download::~Download() {
  // TODO: Abort libwww download
}
//______________________________________________________________________

namespace {

  inline Download* getDownloadObj(HTRequest* request) {
    return static_cast<Download*>(HTRequest_context(request));
  }
  inline Download* getDownloadObj(HTStream* stream) {
    return reinterpret_cast<Download*>(stream);
  }

}
//________________________________________

void Download::run() {
  Assert(output != 0);

  /* The code below (e.g. in putChar()) silently assumes that the
     first data member's address of a Download object is identical to
     the object's address. The C++ standard makes no guarantee about
     this. :-/ */
  Assert(static_cast<void*>(this) == static_cast<void*>(&vptr));
  HTStream* writer = reinterpret_cast<HTStream*>(this); // Shudder... :-)
  request = HTRequest_new();

  // Store within the HTRequest object a ptr to the corresponding Download
  HTRequest_setContext(request, static_cast<void*>(this));

  HTRequest_setOutputFormat(request, WWW_SOURCE); // Raw data, no headers...
  HTRequest_setOutputStream(request, writer); // is sent to writer
  //HTRequest_setDebugStream(request, writer); // body different from 200 OK
  HTAlert_add(alertCallback, HT_A_PROGRESS); // Progress reports
  HTRequest_setAnchor(request, HTAnchor_findAddress(uriVal.c_str()));

  //   HTRequest_addAfter(request, after_load_to_file, NULL, data,
  //                      HT_ALL, HT_FILTER_LAST, FALSE);

  //   BOOL HTRequest_addRange       (HTRequest * request,
  //                                  char * unit, char * range);

  // TODO error handling
  if (HTLoad(request, NO) == NO)
    HTRequest_delete(request);

  return;
}
//______________________________________________________________________

/* Implementation for the libwww HTStream functionality - forwards the
   calls to the Output object.
   Return codes: HT_WOULD_BLOCK, HT_ERROR, HT_OK, >0 to pass back. */

int Download::flush(HTStream*) { return HT_OK; }
int Download::free(HTStream*) { return HT_OK; }
int Download::abort(HTStream*, HTList*) { return HT_OK; }

int Download::putChar(HTStream* me, char c) {
  Download* self = getDownloadObj(me);
  self->totalSize += 1;
  self->output->newData(reinterpret_cast<const byte*>(&c),
                        1, self->totalSize);
  return HT_OK; //TODO: allow newData to pass back error...
}
int Download::putString(HTStream* me, const char* s) {
  Download* self = getDownloadObj(me);
  size_t len = strlen(s);
  self->totalSize += len;
  self->output->newData(reinterpret_cast<const byte*>(s),
                        len, self->totalSize);
  return HT_OK;
}
int Download::write(HTStream* me, const char* s, int l) {
  Download* self = getDownloadObj(me);
  size_t len = static_cast<size_t>(l);
  self->totalSize += len;
  self->output->newData(reinterpret_cast<const byte*>(s),
                        len, self->totalSize);
  return HT_OK;
}
//______________________________________________________________________

// Function which is called by libwww whenever anything happens for a request
BOOL Download::alertCallback(HTRequest* request, HTAlertOpcode op,
                             int /*msgnum*/, const char* /*dfault*/,
                             void* input, HTAlertPar* /*reply*/) {
  if (request == 0) return NO;
  // A Download object hides behind the output stream registered with libwww
  Download* self = getDownloadObj(request);
  char* host = "host";
  if (input != 0) host = static_cast<char*>(input);

  switch (op) {
  case HT_PROG_DNS:
    self->output->info(subst(_("Looking up %1"), host));
    break;
  case HT_PROG_CONNECT:
    self->output->info(subst(_("Contacting %1"), host));
    break;
  case HT_PROG_LOGIN:
    self->output->info(_("Logging in"));
    break;
  case HT_PROG_READ: {
    long len = HTAnchor_length(HTRequest_anchor(request));
    if (len != -1 && static_cast<uint64>(len) != self->totalSize)
      self->output->dataSize(len);
    break;
  }
  case HT_PROG_DONE:
    self->output->finished();
    break;
  case HT_PROG_INTERRUPT:
    self->output->info(_("Interrupted"));
    break;
  case HT_PROG_TIMEOUT:
    self->output->info(_("Timeout"));
    break;
  default:
    break;
  }

  return YES; // Value only relevant for op == HT_A_CONFIRM
}
