/* $Id: sstream.hh,v 1.1 2002/01/25 11:14:47 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2002 Richard Atterer
  | \/|  <richard@atterer.net>
   '` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

  Compatibility layer in case of missing <sstream>

  Because <sstream> was added to the ISO C++ standard very late, some
  compilers do not support its features yet. Below is a minimal
  implementation which supports only the features that jigdo uses.

  Careful - because there appears to be no portable way of making our
  replacement stringstream ignore e.g. "s << setprecision(1)" (because
  according to Stroustrup 21.4.6.1, the exact type returned by
  setprecision() is impl-defined), you must always use s.precision(1)
  or s.setf(ios::fixed). setprecision() and setiosflags() are #defined
  to nonsense below to prevent that error.

*/

#ifndef SSTREAM_HH
#define SSTREAM_HH

#define setprecision dont_use_setprecision__see_top_of_sstream_hh
#define setiosflags dont_use_setiosflags__see_top_of_sstream_hh

#include <config.h>
#if HAVE_SSTREAM
#include <sstream>
namespace std { }
using namespace std;

#else
//______________________________________________________________________

#include <iostream>
#include <string>
namespace std { }
using namespace std;
#include <stdio.h>

class ostringstream : public ostream {
public:
  ostringstream() : val() { }
  const string& str() const { return val; }
  void precision(int) const { }
  void setf(int) const { }
  ostringstream& operator<<(unsigned char x) { return put(x); }
  ostringstream& operator<<(char x) { return put(x); }
  ostringstream& operator<<(int x) { return put(x); }
  ostringstream& operator<<(unsigned x) { return put(x); }
  ostringstream& operator<<(long x) { return put(x); }
  ostringstream& operator<<(unsigned long x) { return put(x); }
  ostringstream& operator<<(uint64 x) { return put(x); }
  ostringstream& operator<<(const char* x) { return put(x); }
  ostringstream& operator<<(const string& x) { return put(x); }
  ostringstream& operator<<(float x) { return put(x); }
  ostringstream& operator<<(double x) { return put(x); }
  inline ostringstream& put(unsigned char x);
  inline ostringstream& put(char x);
  ostringstream& put(int x);
  ostringstream& put(unsigned x);
  ostringstream& put(long x);
  ostringstream& put(unsigned long x);
  ostringstream& put(uint64 x);
  ostringstream& put(double x);
  inline ostringstream& put(const char* x);
  inline ostringstream& put(const string& x);
private:
  string val;
  // Static buffer of fixed size, for snprintf
  static const size_t BUF_SIZE = 1024; // including terminating 0
  static char buf[BUF_SIZE];
};
//________________________________________

ostringstream& ostringstream::put(unsigned char x) {
  val += static_cast<char>(x);
  return *this;
}
ostringstream& ostringstream::put(char x) {
  val += x;
  return *this;
}
ostringstream& ostringstream::put(const char* x) {
  val += x;
  return *this;
}
ostringstream& ostringstream::put(const string& x) {
  val += x;
  return *this;
}
//______________________________________________________________________

// Support functions to allow things like "b64Stream << endl;"

inline ostringstream& flush(ostringstream& s) { return s; }
inline ostringstream& endl(ostringstream& s) { s << '\n'; return s; }
inline ostringstream& operator<<(ostringstream& s,
                                 ostringstream& (*m)(ostringstream&)) {
  return (*m)(s);
}
//______________________________________________________________________

#endif /* HAVE_SSTREAM */
#endif /* defined SSTREAM_HH */
