/* $Id: string.cc,v 1.9 2002/01/25 11:14:47 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2001 Richard Atterer
  | \/|  <richard@atterer.net>
   '` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

*/

#include <stdarg.h>

#include <sstream.hh>
#include <string.hh>
//______________________________________________________________________

/* Convert an unsigned integer n to decimal chars and append to a
   string. If width!=0, pad the number with character "pad" to the
   specified width. */
string& append(string& s, size_t n, size_t width, char pad) {
  const int BUF_LEN = 40; // Enough room for 128-bit integers. :-)
  static char buf[BUF_LEN];
  char* p = buf + BUF_LEN;
  *--p = '\0';
  if (n == 0) {
    *--p = '0';
  } else {
    while (n != 0) {
      *--p = '0' + (n % 10);
      n /= 10;
    }
  }
  for (int i = width - (buf + BUF_LEN - 1 - p); i > 0; --i)
    s += pad;
  return s += p;
}
//______________________________________________________________________

string Subst::subst(const char* format, int args, const Subst arg[]) {
  // Create output
  ostringstream result;
  const char* i = format - 1;
  char max = '1' + args;
  while (*++i != '\0') {
    if (*i != '%') { result << *i; continue; }
    ++i;
    if (*i == '\0') break;
    if (*i == '%') { result << '%'; continue; }
    if (*i >= '1' && *i < max) {
      int n = *i - '1';
      switch (arg[n].type) {
        case INT:      result << arg[n].val.intVal; break;
        case UNSIGNED: result << arg[n].val.unsignedVal; break;
        case LONG:     result << arg[n].val.longVal; break;
        case ULONG:    result << arg[n].val.ulongVal; break;
        case UINT64_P: result << *arg[n].val.uint64Ptr; break;
        case CHAR:     result << arg[n].val.charVal; break;
        case CHAR_P:   result << arg[n].val.charPtr; break;
        case STRING_P: result << *arg[n].val.stringPtr; break;
        default: break;
      }
      continue;
    }
    result << '%' << *i;
  }

  return result.str();
}
