/* $Id: string.hh,v 1.8 2002/01/01 19:00:59 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2001 Richard Atterer
  | \/|  <richard@atterer.net>
   '` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

*/

#ifndef STRING_HH
#define STRING_HH

#include <config.h> /* for size_t */
#include <string>

/** Convert an unsigned integer n to decimal chars and append to a
    string. If width!=0, pad the number with character "pad" to the
    specified width. */
string& append(string& s, size_t n, size_t width = 0, char pad = ' ');
//______________________________________________________________________

class Subst {
public:
  Subst(int x)           { type = INT;      val.intVal = x; }
  Subst(unsigned x)      { type = UNSIGNED; val.unsignedVal = x; }
  Subst(long x)          { type = LONG;     val.longVal = x; }
  Subst(unsigned long x) { type = ULONG;    val.ulongVal = x; }
  Subst(const uint64& x) { type = UINT64_P; val.uint64Ptr = &x; }
  Subst(char x)          { type = CHAR;     val.charVal = x; }
  Subst(const char* x)   { type = CHAR_P;   val.charPtr = x; }
  Subst(const string& x) { type = STRING_P; val.stringPtr = &x; }
  Subst(const string* x) { type = STRING_P; val.stringPtr = x; }
  static string subst(const char* format, int args, const Subst arg[]);
private:
  enum {
    INT, UNSIGNED, LONG, ULONG, UINT64_P, CHAR, CHAR_P, STRING_P
  } type;
  union {
    int intVal;
    unsigned unsignedVal;
    long longVal;
    unsigned long ulongVal;
    const uint64* uint64Ptr;
    char charVal;
    const char* charPtr;
    const string* stringPtr;
  } val;
};
//______________________________________________________________________

/* Example:
   cout << subst("file `%1' not found: %2",
                 string("foo"), strerror(errno)); */

inline string subst(const char* format, Subst a) {
  return Subst::subst(format, 1, &a);
}
inline string subst(const char* format, Subst a, Subst b) {
  Subst arg[] = { a, b };
  return Subst::subst(format, 2, arg);
}
inline string subst(const char* format, Subst a, Subst b, Subst c) {
  Subst arg[] = { a, b, c };
  return Subst::subst(format, 3, arg);
}
inline string subst(const char* format, Subst a, Subst b, Subst c, Subst d) {
  Subst arg[] = { a, b, c, d };
  return Subst::subst(format, 4, arg);
}
inline string subst(const char* format, Subst a, Subst b, Subst c, Subst d,
                    Subst e) {
  Subst arg[] = { a, b, c, d, e };
  return Subst::subst(format, 5, arg);
}
inline string subst(const char* format, Subst a, Subst b, Subst c, Subst d,
                    Subst e, Subst f) {
  Subst arg[] = { a, b, c, d, e, f };
  return Subst::subst(format, 6, arg);
}
inline string subst(const char* format, Subst a, Subst b, Subst c, Subst d,
                    Subst e, Subst f, Subst g) {
  Subst arg[] = { a, b, c, d, e, f, g };
  return Subst::subst(format, 7, arg);
}
inline string subst(const char* format, Subst a, Subst b, Subst c, Subst d,
                    Subst e, Subst f, Subst g, Subst h) {
  Subst arg[] = { a, b, c, d, e, f, g, h };
  return Subst::subst(format, 8, arg);
}
inline string subst(const char* format, Subst a, Subst b, Subst c, Subst d,
                    Subst e, Subst f, Subst g, Subst h, Subst i) {
  Subst arg[] = { a, b, c, d, e, f, g, h, i };
  return Subst::subst(format, 9, arg);
}

#endif
